"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const apollo_server_express_1 = require("apollo-server-express");
const GraphQLLoader_1 = __importDefault(require("./GraphQLLoader"));
const DatabaseLoader_1 = __importDefault(require("./DatabaseLoader"));
class Application {
    constructor(config) {
        this.config = config;
        this.gqlLoader = new GraphQLLoader_1.default();
        this.dbLoader = new DatabaseLoader_1.default(config);
    }
    async buildServer() {
        const { connection, repositories } = await this.dbLoader.getDbConnector();
        return new apollo_server_express_1.ApolloServer({
            playground: true,
            introspection: true,
            typeDefs: await this.gqlLoader.getTypeDefs(),
            resolvers: await this.gqlLoader.getResolvers(),
            uploads: false,
            context: () => ({
                db: {
                    connection,
                    repositories,
                },
            }),
        });
    }
}
exports.default = Application;
//# sourceMappingURL=Application.js.map