"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const typeorm_1 = require("typeorm");
const path_1 = __importDefault(require("path"));
const logger_1 = require("../logger");
const paths_1 = __importDefault(require("./constants/paths"));
class DatabaseConnector {
    constructor(config) {
        this.config = config;
        this.modelsFolder = paths_1.default.ENTITIES;
    }
    async getDbConnector() {
        const connection = await this.createConnection();
        const repositories = await this.loadModels(connection);
        const context = this.constructor.name;
        logger_1.logger.log(`Loaded entities: ${Object.keys(repositories)}`, context);
        return {
            connection,
            repositories,
        };
    }
    async createConnection() {
        const entitiesResolver = path_1.default.join(this.modelsFolder, '*.js');
        const { db: { database, host, password, port, username, ssl, }, } = this.config;
        return typeorm_1.createConnection({
            type: 'postgres',
            host,
            port,
            username,
            password,
            database,
            ssl,
            entities: [
                path_1.default.normalize(entitiesResolver),
            ],
        });
    }
    async loadModels(connection) {
        let modelFiles;
        try {
            modelFiles = fs_1.default.readdirSync(this.modelsFolder)
                .filter((file) => /.js$/.test(file))
                .map(async (modelFile) => Promise.resolve().then(() => __importStar(require(`${this.modelsFolder}/${modelFile}`))));
        }
        catch (e) {
            if ((e === null || e === void 0 ? void 0 : e.code) !== 'ENOENT') {
                logger_1.logger.error(e, null, this.constructor.name);
            }
            return {};
        }
        const loadedEntities = await Promise.all(modelFiles);
        return loadedEntities.reduce((prev, next) => {
            const value = Object.values(next)[0];
            const { name } = value;
            return Object.assign(Object.assign({}, prev), { [name]: connection.getRepository(value) });
        }, {});
    }
}
exports.default = DatabaseConnector;
//# sourceMappingURL=DatabaseLoader.js.map