"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const apollo_server_express_1 = require("apollo-server-express");
const logger_1 = require("../logger");
const paths_1 = __importDefault(require("./constants/paths"));
class GraphqlLoader {
    constructor() {
        this.resolversPath = paths_1.default.RESOLVERS;
        this.schemaPath = paths_1.default.SCHEMA;
        this.resolvers = {};
        this.getResolverName = (file) => path_1.default.parse(file).name;
    }
    async loadResolversFromDir(folder) {
        const normalizedPath = path_1.default.join(this.resolversPath, folder);
        let files;
        try {
            files = fs_1.default.readdirSync(normalizedPath).filter((file) => /.js$/.test(file));
        }
        catch (e) {
            if ((e === null || e === void 0 ? void 0 : e.code) !== 'ENOENT') {
                logger_1.logger.error(e, null, this.constructor.name);
            }
            return null;
        }
        const resolversArray = await Promise.all(files.map(async (it) => ({
            [this.getResolverName(it)]: await Promise.resolve().then(() => __importStar(require(path_1.default.join(normalizedPath, it)))),
        })));
        const reducedResolversObject = resolversArray.reduce((prev, next) => {
            const name = Object.keys(next)[0];
            const value = Object.values(next)[0];
            if (!value.default) {
                return Object.assign({}, prev);
            }
            return Object.assign(Object.assign({}, prev), { [name]: value.default });
        }, {});
        if (Object.keys(reducedResolversObject).length) {
            return reducedResolversObject;
        }
        return null;
    }
    async loadResolversFromFile() {
        const resolversFile = await Promise.resolve().then(() => __importStar(require(path_1.default.join(this.resolversPath, 'resolvers.js'))));
        this.resolvers.Query = Object.assign(Object.assign({}, this.resolvers.Query), resolversFile.default.Query);
        this.resolvers.Mutation = Object.assign(Object.assign({}, this.resolvers.Mutation), resolversFile.default.Mutation);
    }
    async loadResolvers() {
        const query = await this.loadResolversFromDir('queries');
        const mutation = await this.loadResolversFromDir('mutations');
        if (query) {
            this.resolvers.Query = query;
        }
        if (mutation) {
            this.resolvers.Mutation = mutation;
        }
        await this.loadResolversFromFile();
    }
    async getResolvers() {
        await this.loadResolvers();
        const context = this.constructor.name;
        logger_1.logger.log(`Loaded queries: ${Object.keys(this.resolvers.Query)}`, context);
        logger_1.logger.log(`Loaded mutations: ${Object.keys(this.resolvers.Mutation)}`, context);
        return this.resolvers;
    }
    getTypeDefs() {
        return apollo_server_express_1.gql `${fs_1.default.readFileSync(this.schemaPath, 'utf8')}`;
    }
}
exports.default = GraphqlLoader;
//# sourceMappingURL=GraphQLLoader.js.map